Package: DSS
Title: Dispersion shrinkage for sequencing data
Version: 2.60.0
Date: 2026-4-9
Author: Hao Wu<hao.wu@emory.edu>, Hao Feng<hxf155@case.edu>
Depends: R (>= 3.5.0), methods, Biobase, BiocParallel, bsseq, parallel
Imports: utils, graphics, stats, splines
Maintainer: Hao Wu <hao.wu@emory.edu>, Hao Feng <hxf155@case.edu>
Description: DSS is an R library performing differntial analysis for
        count-based sequencing data. It detectes differentially
        expressed genes (DEGs) from RNA-seq, and differentially
        methylated loci or regions (DML/DMRs) from bisulfite sequencing
        (BS-seq). The core of DSS is a new dispersion shrinkage method
        for estimating the dispersion parameter from Gamma-Poisson or
        Beta-Binomial distributions.
License: GPL
VignetteBuilder: knitr
Suggests: BiocStyle, knitr, rmarkdown, edgeR
biocViews: Sequencing, RNASeq, DNAMethylation,GeneExpression,
        DifferentialExpression,DifferentialMethylation
Config/pak/sysreqs: make libbz2-dev liblzma-dev libxml2-dev libssl-dev
        xz-utils zlib1g-dev
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 12:36:36 UTC
RemoteUrl: https://github.com/bioc/DSS
RemoteRef: RELEASE_3_23
RemoteSha: a954918494c51ec343aa885105c0b80ec69e4631
NeedsCompilation: yes
Packaged: 2026-04-29 13:50:48 UTC; root
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-29 13:55:50 UTC; unix
