useDynLib(DSS)

import(methods)
import(utils)
import(graphics)
import(stats)
importFrom(splines, ns)
import(Biobase)
import(BiocParallel)
import(bsseq)

exportClasses("SeqCountSet")
exportMethods(
              normalizationFactor, "normalizationFactor<-",
              dispersion, "dispersion<-"
              )

export(
       newSeqCountSet,
       estDispersion,
       estNormFactors,
       waldTest,
       DSS.DE,
       DMLtest,
       callDML,
       callDMR,
       makeBSseqData,
       showOneDMR,
       DMLfit.multiFactor,
       DMLtest.multiFactor
)

